<?php
namespace Neklo\Scheduler\Observer;

use Magento\Framework\Console\CLI;
use Neklo\Scheduler\Api\Data\JobInterface;
use Neklo\Scheduler\Model\ResourceModel\Job\CollectionFactory;
use Magento\Cron\Model\Schedule;

class Observer extends \Magento\Cron\Observer\ProcessCronQueueObserver
{
    protected $jobCollectionFactory;
    protected $jobFactory;

    public function __construct(
        \Magento\Framework\ObjectManagerInterface $objectManager,
        \Magento\Cron\Model\ScheduleFactory $scheduleFactory,
        \Magento\Framework\App\CacheInterface $cache,
        \Magento\Cron\Model\ConfigInterface $config,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\App\Console\Request $request,
        \Magento\Framework\ShellInterface $shell,
        \Magento\Framework\Stdlib\DateTime\DateTime $timezone,
        \Magento\Framework\Process\PhpExecutableFinderFactory $phpExecutableFinderFactory,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\App\State $state,
        CollectionFactory $jobCollectionFactory,
        \Neklo\Scheduler\Model\JobFactory $jobFactory
    ) {
        parent::__construct($objectManager, $scheduleFactory, $cache, $config,
            $scopeConfig, $request, $shell, $timezone, $phpExecutableFinderFactory,$logger,$state);
        $this->jobCollectionFactory = $jobCollectionFactory;
        $this->jobFactory = $jobFactory;
    }

    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        $pendingJobs = $this->_getPendingSchedules();
        $currentTime = $this->timezone->scopeTimeStamp();
        $reqGroup = $this->_request->getParam('group');
        /* @var $jobs \Neklo\Scheduler\Model\ResourceModel\Job\Collection */
        $jobs = $this->jobCollectionFactory->create();
//        $jobs->addFieldToFilter(JobInterface::JOB_ENABLED, 1);
        $jobGroupsRoot = $this->groupJobs($jobs);

        foreach ($jobGroupsRoot as $groupId => $jobsRoot) {
            if ($reqGroup !== null
                && $reqGroup !== '\'' . ($groupId) . '\''
                && $reqGroup !== $groupId) {
                continue;
            }
            if ($this->_request->getParam(self::STANDALONE_PROCESS_STARTED) !== '1' &&
                $this->_scopeConfig->getValue('system/cron/' . $groupId . '/use_separate_process', \Magento\Store\Model\ScopeInterface::SCOPE_STORE) == 1
            ) {
                $this->_shell->execute(
                    'php %s cron:run --group=' . $groupId
                    . ' --' . CLI::INPUT_KEY_BOOTSTRAP . '='
                    . self::STANDALONE_PROCESS_STARTED . '=1',
                    [
                        BP . '/bin/magento'
                    ]
                );
                continue;
            }

            foreach ($pendingJobs as $schedule) {
                $jobConfig = isset($jobsRoot[$schedule->getJobCode()])
                    ? $jobsRoot[$schedule->getJobCode()] : null;
                if (!$jobConfig) {
                    continue;
                } else

                if (!$jobConfig->isEnabled()) {
                    $schedule->setStatus(Schedule::STATUS_ERROR)
                        ->setMessages('Run disabled by job config')->save();
                    continue;
                }

                $scheduledTime = strtotime($schedule->getScheduledAt());
                if ($scheduledTime > $currentTime) {
                    continue;
                }

                try {
                    if ($schedule->tryLockJob()) {
                        $this->_runJob($scheduledTime, $currentTime,
                            $jobConfig->getData(), $schedule, $groupId);
                    }
                } catch (\Exception $e) {
                    $schedule->setMessages($e->getMessage());
                }
                $schedule->save();
            }

            $this->_generate($groupId);
//            $this->_cleanup($groupId);
        }

        $this->updateJobs();
    }

    /**
     *
     * @param \Neklo\Scheduler\Model\ResourceModel\Job\Collection $jobs
     * @return array
     */
    protected function groupJobs(
        \Neklo\Scheduler\Model\ResourceModel\Job\Collection $jobs
    ) {
        $ret = [];
        foreach ($jobs->getItems() as $job) {
            if (!isset($ret[$job->getGroup()])) {
                $ret[$job->getGroup()] = [];
            }
            $ret[$job->getGroup()][$job->getName()] = $job;
        }
        return $ret;
    }

    protected function updateJobs()
    {
        $jobGroups = $this->_config->getJobs();
        foreach($jobGroups as $group => $jobs) {
            foreach($jobs as $jobName => $job) {
                $model = $this->jobFactory->create();
                $model = $model->load($jobName, JobInterface::JOB_NAME);
                if (!$model->getId()) {
                    $model->setName($jobName)
                        ->setMethod($job[JobInterface::JOB_METHOD])
                        ->setInstance($job[JobInterface::JOB_CLASS])
                        ->setEnabled(true)
                        ->setGroup($group);
                    if (isset($job[JobInterface::JOB_SCHEDULE])) {
                        $model->setSchedule($job[JobInterface::JOB_SCHEDULE]);
                    }
                    if (isset($job[JobInterface::JOB_CONFIG])) {
                        $model->setConfigPath($job[JobInterface::JOB_CONFIG]);
                    }
                    $model->save();
                }
            }
        }
    }
}
