<?php

namespace Neklo\Scheduler\Setup;

use Magento\Eav\Setup\EavSetup;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Setup\InstallDataInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Neklo\Scheduler\Api\Data\JobInterface;

/**
 * @codeCoverageIgnore
 */
class InstallData implements InstallDataInterface
{
    /**
     *
     * @var \Neklo\Scheduler\Helper\Job
     */
    protected $jobHelper;

    /**
     *
     * @var \Neklo\Scheduler\Model\JobFactory
     */
    protected $jobFactory;

    public function __construct(
        \Neklo\Scheduler\Helper\Job $helper,
        \Neklo\Scheduler\Model\JobFactory $factory
    ) {
        $this->jobHelper = $helper;
        $this->jobFactory = $factory;
    }
    /**
     * {@inheritdoc}
     */
    public function install(
        ModuleDataSetupInterface $setup,
        ModuleContextInterface $context
    ) {
        $jobs = $this->jobHelper->getJobs();
        if (is_array($jobs)) {
            foreach ($jobs as $group => $groupJobs) {
                foreach ($groupJobs as $jobName => $job) {
                    $item = $this->jobFactory->create();
                    $item->load($jobName, JobInterface::JOB_NAME);
                    $item
                        ->setName($jobName)
                        ->setMethod($job[JobInterface::JOB_METHOD])
                        ->setInstance($job[JobInterface::JOB_CLASS])
                        ->setEnabled(true)
                        ->setGroup($group);
                    if (isset($job[JobInterface::JOB_SCHEDULE])) {
                        $item->setSchedule($job[JobInterface::JOB_SCHEDULE]);
                    }
                    if (isset($job[JobInterface::JOB_CONFIG])) {
                        $item->setConfigPath($job[JobInterface::JOB_CONFIG]);
                    }
                    $item->save();
                }
            }
        }
    }
}